!
! $Id: modupdate.F 779 2007-12-22 17:04:17Z rblod $
!
!     AGRIF (Adaptive Grid Refinement In Fortran)
!
!     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
!                        Christophe Vouland (Christophe.Vouland@imag.fr)
!
!     This program is free software; you can redistribute it and/or modify
!     it under the terms of the GNU General Public License as published by
!     the Free Software Foundation; either version 2 of the License, or
!     (at your option) any later version.
!
!     This program is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!     GNU General Public License for more details.
!
!     You should have received a copy of the GNU General Public License
!     along with this program; if not, write to the Free Software
!     Foundation, Inc., 59 Temple Place- Suite 330, Boston, MA 02111-1307, USA.
!
!---------------------------------------------------------------------------------------------------
!> Module Agrif_User_Interpolation
!!
!! This module contains procedures used by AGRIF users to do an interpolation 
!! either within the domain or at boundary or on the whole domain.
!---------------------------------------------------------------------------------------------------
!
module Agrif_User_Interpolation
!
  use Agrif_Boundary
  use Agrif_user_variables
!  
  Implicit none

contains
!
!===================================================================================================
!  subroutine Agrif_Set_bc
!> This subroutine is used to specify the variable we want to interpolate and the boundary location of 
!! the current grid we want to interpolate (boundary interpolation).  
!---------------------------------------------------------------------------------------------------
subroutine Agrif_Set_bc ( tabvarsindic, bcinfsup, Interpolationshouldbemade )
!---------------------------------------------------------------------------------------------------
    integer,               intent(in)   :: tabvarsindic !< index of the variable in tabvars
    integer, dimension(2), intent(in)   :: bcinfsup     !< location
    logical, optional,     intent(in)   :: Interpolationshouldbemade !< interpolation should be made
!
    integer                         :: indic ! indice of the variable in tabvars
    type(Agrif_Variable),  pointer  :: var
!
    var => Agrif_Search_Variable(Agrif_Curgrid,tabvarsindic)
    if (.not.associated(var)) return ! Grand mother grid case
!
    if ( Agrif_Curgrid % fixedrank /= 0 ) then
        if ( .not.associated(var % oldvalues2D) ) then
            allocate(var % oldvalues2D(2,1))
            var % interpIndex = -1
            var % oldvalues2D = 0.
        endif
        if ( present(Interpolationshouldbemade) ) then
            var % Interpolationshouldbemade = Interpolationshouldbemade
        endif
    endif
!
    var % bcinf = bcinfsup(1)
    var % bcsup = bcinfsup(2)
!---------------------------------------------------------------------------------------------------
end subroutine Agrif_Set_bc
!===================================================================================================
!
!===================================================================================================
!  subroutine Agrif_Set_interp
!> This procedure specify the type of interpolation in case of interpolation within the domain.
!! The index of the variable we want to interpolate is a profile defined with the subroutine Agrif_Declare_Variable.
!
! Interp1 indicates the type of interpolation in the first dimension
!---------------------------------------------------------------------------------------------------
subroutine Agrif_Set_interp ( tabvarsindic, interp, interp1, interp2, interp3 , interp4)
!---------------------------------------------------------------------------------------------------
    integer,           intent(in)   :: tabvarsindic !< index of the variable in tabvars
    integer, optional, intent(in)   :: interp, interp1, interp2, interp3, interp4 !< The type of interpolation
!
    integer                         :: indic ! index of the variable in tabvars
    type(Agrif_Variable), pointer   :: var
!
    var => Agrif_Search_Variable(Agrif_Curgrid,tabvarsindic)
    if (.not.associated(var)) return ! Grand mother grid case
!
    var % type_interp = Agrif_Constant
!
    if (present(interp))    var % type_interp    = interp
    if (present(interp1))   var % type_interp(1) = interp1
    if (present(interp2))   var % type_interp(2) = interp2
    if (present(interp3))   var % type_interp(3) = interp3
    if (present(interp4))   var % type_interp(4) = interp4
!---------------------------------------------------------------------------------------------------
end subroutine Agrif_Set_interp
!===================================================================================================
!
!===================================================================================================
!  subroutine Agrif_Set_bcinterp
!> This subroutine is used to indicate the type of interpolation in case of boundary interpolation.
!! The index of the variable we want to interpolate is a profile defined with the subroutine Agrif_Declare_Variable.
!
!> Interp1 indicates the type of interpolation in the first dimension
!! Interp21 indicates the first variable of the second dimension
!---------------------------------------------------------------------------------------------------
subroutine Agrif_Set_bcinterp ( tabvarsindic, interp,   interp1,  interp2,  interp3, interp4, &
                                              interp11, interp12, interp21, interp22 )
!---------------------------------------------------------------------------------------------------
    INTEGER,           intent(in)   :: tabvarsindic !< index of the variable in tabvars
    INTEGER, OPTIONAL, intent(in)   :: interp,   interp1,  interp2,  interp3, interp4 !< type of interpolation
    INTEGER, OPTIONAL, intent(in)   :: interp11, interp12, interp21, interp22 !< dimension we want to interpolate
!
    INTEGER                         :: indic ! index of the variable in tabvars
    TYPE(Agrif_Variable), pointer   :: var
!
    var => Agrif_Search_Variable(Agrif_Curgrid,tabvarsindic)
!
    var % type_interp_bc = Agrif_Constant
!
    if (present(interp))    var % type_interp_bc      = interp
    if (present(interp1))   var % type_interp_bc(:,1) = interp1
    if (present(interp11))  var % type_interp_bc(1,1) = interp11
    if (present(interp12))  var % type_interp_bc(1,2) = interp12
    if (present(interp2))   var % type_interp_bc(:,2) = interp2
    if (present(interp21))  var % type_interp_bc(2,1) = interp21
    if (present(interp22))  var % type_interp_bc(2,2) = interp22
    if (present(interp3))   var % type_interp_bc(:,3) = interp3
    if (present(interp4))   var % type_interp_bc(:,4) = interp4
!---------------------------------------------------------------------------------------------------
end subroutine Agrif_Set_bcinterp
!===================================================================================================
!
!===================================================================================================
!  subroutine Agrif_Init_variable
!> This subroutine is used for interpolation over the whole domain (within the domain and boundaries). 
!! It specifies the variable we want to interpolate and define through an input procedure (procname) 
!! what we do either on parent or child grid. The index of the variable we want to interpolate is a profile
!! defined with the subroutine Agrif_Declare_Variable.
!---------------------------------------------------------------------------------------------------
subroutine Agrif_Init_variable ( tabvarsindic, procname )
!---------------------------------------------------------------------------------------------------
    INTEGER, intent(in)  :: tabvarsindic     !< index of the variable in tabvars
    procedure()          :: procname         !< Data recovery procedure written by users
!
    if ( Agrif_Curgrid%level <= 0 ) return
!
    call Agrif_Interp_variable(tabvarsindic, procname)
    call Agrif_Bc_variable(tabvarsindic, procname, 1.)
!---------------------------------------------------------------------------------------------------
end subroutine Agrif_Init_variable
!===================================================================================================
!
!===================================================================================================
!  subroutine Agrif_Bc_variable
!> This subroutine is used for boundary interpolation. It specifies the variable we want to interpolate
!! and define through an input procedure (procname) what we do either on parent or child grid.
!! The index of the variable we want to interpolate is a profile defined with the subroutine Agrif_Declare_Variable.
!---------------------------------------------------------------------------------------------------
subroutine Agrif_Bc_variable ( tabvarsindic, procname, calledweight )
!---------------------------------------------------------------------------------------------------
    integer,        intent(in) :: tabvarsindic  !< index of the variable in tabvars
    procedure()                :: procname		!< Data recovery procedure written by users
    real, optional, intent(in) :: calledweight	!< weight of interpolation
!
    real    :: weight
    logical :: pweight
    integer :: indic
    integer :: nbdim
    type(Agrif_Variable), pointer :: root_var
    type(Agrif_Variable), pointer :: parent_var
    type(Agrif_Variable), pointer :: child_var
    type(Agrif_Variable), pointer :: child_tmp      ! Temporary variable on the child grid
    integer :: i
    integer,dimension(7) :: lb, ub
!
    if ( Agrif_Curgrid%level <= 0 ) return
!
!
    if ( present(calledweight) ) then
        weight  = calledweight
        pweight = .true.
    else
        weight  = 0.
        pweight = .false.
    endif
!
        child_var  => Agrif_Search_Variable(Agrif_Curgrid,tabvarsindic)
        parent_var => child_var % parent_var
        root_var   => child_var % root_var
!
    nbdim = root_var % nbdim
!
    do i=1,nbdim
      if (root_var%coords(i) == 0) then
        lb(i) = parent_var%lb(i)
        ub(i) = parent_var%ub(i)
      else
        lb(i) = child_var%lb(i)
        ub(i) = child_var%ub(i)
      endif
    enddo

    select case( nbdim )
    case(1)
        allocate(parray1(lb(1):ub(1)))
    case(2)
        allocate(parray2(lb(1):ub(1), &
                         lb(2):ub(2) ))
    case(3)
        allocate(parray3(lb(1):ub(1), &
                         lb(2):ub(2), &
                         lb(3):ub(3) ))
    case(4)
        allocate(parray4(lb(1):ub(1), &
                         lb(2):ub(2), &
                         lb(3):ub(3), &
                         lb(4):ub(4) ))
    case(5)
        allocate(parray5(lb(1):ub(1), &
                         lb(2):ub(2), &
                         lb(3):ub(3), &
                         lb(4):ub(4), &
                         lb(5):ub(5) ))
    case(6)
        allocate(parray6(lb(1):ub(1), &
                         lb(2):ub(2), &
                         lb(3):ub(3), &
                         lb(4):ub(4), &
                         lb(5):ub(5), &
                         lb(6):ub(6) ))
    end select
!
!   Create temporary child variable
    allocate(child_tmp)
!
    child_tmp % root_var => root_var
    child_tmp % oldvalues2D => child_var % oldvalues2D
!
!   Index indicating if a space interpolation is necessary
    child_tmp % interpIndex =  child_var % interpIndex
    child_tmp % list_interp => child_var % list_interp
    child_tmp % Interpolationshouldbemade = child_var % Interpolationshouldbemade
!
    child_tmp % point = child_var % point
    child_tmp % lb = child_var % lb
    child_tmp % ub = child_var % ub

    child_tmp % lubglob = child_var % lubglob

!
    child_tmp % bcinf = child_var % bcinf
    child_tmp % bcsup = child_var % bcsup
!
    child_tmp % childarray = child_var % childarray
    child_tmp % memberin   = child_var % memberin
!
    call Agrif_CorrectVariable(parent_var, child_tmp, pweight, weight, procname)
!
    child_var % childarray = child_tmp % childarray
    child_var % memberin   = child_tmp % memberin
!
    child_var % oldvalues2D => child_tmp % oldvalues2D
    child_var % list_interp => child_tmp % list_interp
!
    child_var % interpIndex = child_tmp % interpIndex
!
    deallocate(child_tmp)
!
    select case( nbdim )
        case(1); deallocate(parray1)
        case(2); deallocate(parray2)
        case(3); deallocate(parray3)
        case(4); deallocate(parray4)
        case(5); deallocate(parray5)
        case(6); deallocate(parray6)
    end select
!---------------------------------------------------------------------------------------------------
end subroutine Agrif_Bc_variable
!===================================================================================================
!
!===================================================================================================
!  subroutine Agrif_Interp_variable
!> This subroutine is used for interpolation within the domain. It specifies the variable we want to interpolate
!! and define through an input procedure (procname) what we do either on parent or child grid.
!! The index of the variable we want to interpolate is a profile defined with the subroutine Agrif_Declare_Variable.
!---------------------------------------------------------------------------------------------------
subroutine Agrif_Interp_variable ( tabvarsindic, procname )
!---------------------------------------------------------------------------------------------------
    integer,     intent(in)     :: tabvarsindic     !< index of the variable in tabvars
    procedure()                 :: procname         !< Data recovery procedure written by users
!
    integer :: nbdim
    integer :: indic  ! indice of the variable in tabvars
    logical :: torestore
    type(Agrif_Variable), pointer   :: root_var
    type(Agrif_Variable), pointer   :: parent_var       ! Variable on the parent grid
    type(Agrif_Variable), pointer   :: child_var        ! Variable on the parent grid
    type(Agrif_Variable), pointer   :: child_tmp        ! Temporary variable on the child grid
!
    if ( Agrif_Curgrid%level <= 0 ) return
!

        child_var  => Agrif_Search_Variable(Agrif_Curgrid,tabvarsindic)
        parent_var => child_var % parent_var
        root_var   => child_var % root_var

!
    nbdim     = root_var % nbdim
    torestore = root_var % restore
!
    allocate(child_tmp)
!
    child_tmp % root_var => root_var
    child_tmp % nbdim = root_var % nbdim
    child_tmp % point = child_var % point
    child_tmp % lb = child_var % lb
    child_tmp % ub = child_var % ub
    child_tmp % lubglob = child_var % lubglob
    child_tmp % interpIndex =  child_var % interpIndex
    child_tmp % list_interp => child_var % list_interp
    child_tmp % Interpolationshouldbemade = child_var % Interpolationshouldbemade
!
    if ( torestore ) then
        select case( nbdim )
        case(1)
            parray1 = child_var % array1
            child_tmp % restore1D => child_var % restore1D
        case(2)
            parray2 = child_var % array2
            child_tmp % restore2D => child_var % restore2D
        case(3)
            parray3 = child_var % array3
            child_tmp % restore3D => child_var % restore3D
        case(4)
            parray4 = child_var % array4
            child_tmp % restore4D => child_var % restore4D
        case(5)
            parray5 = child_var % array5
            child_tmp % restore5D => child_var % restore5D
        case(6)
            parray6 = child_var % array6
            child_tmp % restore6D => child_var % restore6D
        end select
    endif
!
    call Agrif_InterpVariable(parent_var, child_tmp, torestore, procname)
!
    child_var % list_interp => child_tmp % list_interp
!
    deallocate(child_tmp)
!---------------------------------------------------------------------------------------------------
end subroutine Agrif_Interp_variable
!===================================================================================================
!

end module Agrif_User_Interpolation
